package vrmd_api;
use strict;
use IO::Select;
use IO::Socket;
use URI;
#use LWP::UserAgent;
#use HTTP::Request::Common;

use vars qw(@EXPORT @ISA);
@ISA = qw(Exporter);
@EXPORT = qw(getDomainStatus getTLDs getSelectList);

# Version 0.1
# Copyright 2004 by Marten Lehmann, Variomedia IT-Service GmbH

sub doHttpPostRequest {
	my ($uri, $vars) = @_;
	my ($host, $timeout, $start, $timeout_occured);
	my ($content, $length, $uri_obj);
	my ($socket, $select, $bytes, $buffer, $request, $response);
	my ($body, $line, $payload);
	
	$host = "api.variomedia.de";
	$vars->{"userid"} = "";
	$vars->{"password"} = "";
	$timeout = 5;

	$start = time();
	$socket = IO::Socket::INET->new(PeerAddr => $host, PeerPort => 80, Timeout => $timeout, Blocking => 0);
	if ($socket) {
		$uri_obj = URI->new();
		$uri_obj->query_form(%{$vars});
		$content = $uri_obj->query;
		$length = length($content);
		$request = <<EOF;
POST $uri HTTP/1.0
Host: $host
Connection: close
Accept-Encoding: identity
Content-Type: application/x-www-form-urlencoded
Content-Length: $length

$content
EOF
		$timeout_occured = 1;
		syswrite($socket, $request);

		use IO::Select;
		$select = IO::Select->new($socket);

		while (time() - $start <= $timeout) {
			if ($select->can_read(0.1)) {
				$bytes = sysread($socket, $buffer, 1024);
				if ($bytes) {
					$response .= $buffer;
				} else {
					close $socket;
					$timeout_occured = 0;
					last;
				}
			}
		}

#		while ($select->can_read()) {
#			$bytes = sysread($socket, $buffer, 1024);
#			if ($bytes) {
#				$response .= $buffer;
#			} else {
#				close $socket;
#				last;
#			}
#		}

		$response = "" if $timeout_occured;
		$response =~ s/\r\n/\n/g;

		foreach $line (split(/\n/, $response)) {
			if ($line =~ /^$/) {
				$body = 1;
				next;
			}
			next unless $body;
			$payload .= "$line\n";
		}

		return $payload;
	}
	
}

sub getDomainStatus {
	my ($sld, $tld, $utf) = @_;
	my (%status, $uri);
	my ($response, $line, $body, $key, $value);

	$utf ||= "yes";
	$status{"sld"} = $sld;
	$status{"domain"} = "$sld.$tld";
	$status{"sld_form"} = $status{"sld"};
	$status{"sld_form"} =~ s/&/&amp;/g;
	$status{"sld_form"} =~ s/</&lt;/g;
	$status{"sld_form"} =~ s/>/&gt;/g;

	$response = doHttpPostRequest("/getDomainStatus", {"domain" => $status{"domain"}, "utf" => $utf});

	if ($sld) {

		if ($response) {
			$status{"raw"} = $response;

			foreach $line (split(/\n/, $status{"raw"})) {
				next if $line =~ /^#/;
				next if $line =~ /^\s*$/;

				$line =~ /^([^:]+):\s+(.*)$/;
				($key, $value) = ($1, $2);

				if ($key eq "domain-ace") {
					$status{"domain-ace"} = $value;
				} elsif ($key eq "status") {
					$status{"status"} = $value;
				}
			}
		}

		$status{"status"} = "timeout" unless $status{"status"};
	} else {
		$status{"status"} = "invalid";
	}

	if ($status{"domain-ace"}) {
		$status{"domain_full"} = $status{"domain"}. " [". $status{"domain-ace"}. "]";
	} else {
		$status{"domain_full"} = $status{"domain"};
	}

	if ($status{"status"} eq "free") {
		$status{"worded"} = "Die Domain ist noch frei.";
	} elsif ($status{"status"} eq "taken") {
		$status{"worded"} = "Die Domain ist bereits vergeben.";
	} elsif ($status{"status"} eq "invalid") {
		$status{"worded"} = "Die Domain kann nicht registriert werden (ung&uuml;ltige Zeichen, Name zu kurz etc.).";
	} elsif ($status{"status"} eq "timeout") {
		$status{"worded"} = "Der Status der Domain konnte nicht abgefragt werden (Timeout).";
	} elsif ($status{"status"} eq "unsupported") {
		$status{"worded"} = "Domains mit dieser Endung k&ouml;nnen nicht &uuml;ber uns registriert/abgefragt werden.";
	} else {
		$status{"worded"} = "Ein unbekannter Status wurde zur&uuml;ckgeliefert: ". $status{"status"}. ".";
	}

	return \%status;
}

sub getTLDs {
	my ($tld_file, $content, $line, $tld_list);
	$tld_file = "tlds.txt";
	if (-s $tld_file) {
		open(TLDS, $tld_file);
		while (<TLDS>) {
			$content .= $_;
		}
		close(TLDS);
	} else {
		$content = doHttpPostRequest("/getTLDs");
		if ($content) {
			open(TLDS, ">$tld_file");
			print TLDS $content;
			close(TLDS);
		}
	}

	foreach $line (split(/\n/, $content)) {
		next if $line =~ /^#/;
		next if $line =~ /^\s*$/;
		$tld_list .= "$line\n";
	}

	return $tld_list;
}

sub getSelectList {
	my ($choosen_tld) = @_;
	my ($line, $tld, $flags, $tld_list, $select_list);

	$choosen_tld = "de" unless $choosen_tld;
	$tld_list = getTLDs();
	
	foreach $line (split(/\n/, $tld_list)) {
		($tld, $flags) = split(/\t/, $line);
		$select_list .= "<option value=\"$tld\"". ($choosen_tld eq $tld ? " selected" : ""). ">$tld". ($flags =~ /I/ ? " *" : ""). "</option>\n";
	}
	
	return $select_list;
}

1;
